import java.io.*;
import javax.swing.*;

public class ImageToClass {
	
	// Supported file types
	public final static String[] SUPPORTED = {"GIF", "JPG", "JPEG", "PNG"};
	// Graphical user interface
	private final ImageToClassGUI GUI;

	// Main method
	public static void main(String args[]){
		// Assume command line version
		ImageToClass itc = new ImageToClass(false);
		try{
			// No arguments means GUI version
			if(args.length == 0){
				itc = new ImageToClass(true);
			}
			else{
				// Help
				if(args[0].equals("help") || args[0].equals("-help") || args[0].equals("--help") || args[0].equals("/help")) usage();
				// Too many arguments
				if(args.length > 2) usage();
				// Process arguments
				String src = null;
				String dst = null;
				if(supported(args[0]) && !removeExtension(args[0]).equals("")) src = args[0];
				else usage();
				if(args.length == 2) dst = className(removeExtension(args[1])) + ".java";
				else dst = getDefaultDst(src);
				// Convert image
				itc.imageToClass(src, dst);
				Runtime.getRuntime().exit(0);
			}
		}
		// Error if any exceptions arose
		catch(Exception ex){
			itc.error(ex);
		}
	}
	
	// Display usage information
	public static void usage(){
		String message = "\nImageToClass by Stephen Ware\nUsage: java -jar ImageToClass.jar image/path/and/name.ext [destination/path/and/name.java]\nDestination defaults to same directory, same file name.\nSupported formats: ";
		for(int i=0; i<SUPPORTED.length; i++){
			message += SUPPORTED[i];
			if(i+1 < SUPPORTED.length) message += ", ";
			else message += "\n";
		}
		System.out.println(message);
		Runtime.getRuntime().exit(1);
	}
	
	// Simple runnable to execute GUI creation on event dispatching thread
	private class GUImaker implements Runnable {
		private final ImageToClass itc;
		private ImageToClassGUI gui = null;
		private boolean done = false;
		public GUImaker(ImageToClass itc){ this.itc = itc; }
		public void run(){ gui = new ImageToClassGUI(itc); done = true; }
		public ImageToClassGUI getGUI(){ while(!done){ try{ Thread.sleep(1); }catch(InterruptedException ex){} } return gui;  }
	}
	
	// Constructor
	public ImageToClass(boolean gui){
		// Setup GUI, if any
		if(gui){
			GUImaker guimaker = new GUImaker(this);
			SwingUtilities.invokeLater(guimaker);
			GUI = guimaker.getGUI();
		}
		else GUI = null;
	}
	
	// Alert the user
	public void alert(String message, int icon){
		// GUI
		if(GUI != null){
			JOptionPane.showMessageDialog(GUI, message, "Alert", icon);
		}
		// Command line
		else{
			System.out.println(message);
		}
	}
	
	// Convenience alert
	public void alert(String message){
		alert(message, JOptionPane.INFORMATION_MESSAGE);
	}
	
	// Handle errors
	public void error(String message, Exception ex){
		if(message == null && ex != null){
			// Unwind stack trace
			StringWriter sw = new StringWriter();
			ex.printStackTrace(new PrintWriter(sw));
			message = sw.toString();
		}
		// Alert
		alert(message, JOptionPane.ERROR_MESSAGE);
		if(GUI == null) Runtime.getRuntime().exit(-1);
	}
	
	// Convenience error
	public void error(Exception ex){
		error(null, ex);
	}
	
	// Convenience error
	public void error(String message){
		error(message, null);
	}
	
	// Convert image to java code
	public void imageToClass(String srcPath, String dstPath){
		// Convert source image into byte array
		File srcFile = new File(srcPath);
		if(srcFile.length() > Integer.MAX_VALUE){ error("The source file is too large."); return; }
		byte[] bytes = new byte[0];
		try{
			bytes = getStreamAsBytes(new FileInputStream(srcFile), (int)srcFile.length());
			if(bytes == null) return;
		}
		catch(FileNotFoundException ex){ error("The source file does not exist."); return; }
		// Write to destination file
		File dstFile = new File(dstPath);
		try{
			BufferedWriter out = new BufferedWriter(new FileWriter(dstFile));
			// Imports
			out.write("import javax.swing.ImageIcon;\n");
			// Credit comments and JavaDoc
			out.write("/** <p>A subclass of javax.swing.ImageIcon with the data from " + srcFile.getName() + " hard-coded into it.</p>\n");
			out.write(" * <p>Created with the <a href=\"http://stephengware.com/projects/imagetoclass\">ImageToClass tool</a>, by Stephen G. Ware.</p>\n");
			out.write(" * @author Stephen G. Ware */\n");
			// Header
			out.write("public class " + removeExtension(dstFile.getName()) + " extends ImageIcon {\n");
			out.write("\tprivate static final long serialVersionUID = " + bytes.length + ";\n");
			// Byte array(s)
			int chunk = 1024;
			int length = 0;
			int numArrays = 0;
			double progress;
			while(length < bytes.length){
				out.write("\tprivate static byte[] data" + numArrays + "(){ return new byte[] {");
				for(int i=0; ((i<chunk)&&(length<bytes.length)); i++){
					out.write(Integer.toString((int)(bytes[length])));
					if(i+1<chunk && length+1<bytes.length) out.write(",");
					length++;
					// Progress (if GUI)
					progress = ((double)(length)) / ((double)(bytes.length)) * 0.5;
					if(progress > 0.5) progress = 0.5;
					if(GUI != null) GUI.setProgress(0.5 + progress);
				}
				out.write("}; }\n");
				numArrays++;
			}
			// Glue method
			out.write("\tprivate static byte[] getData(){\n");
			out.write("\t\tbyte[] data = new byte[" + bytes.length + "];\n");
			for(int i=0; i<numArrays; i++){
				out.write("\t\tSystem.arraycopy(data" + i + "(), 0, data, " + (i * chunk) + ", " + Math.min(chunk, (bytes.length - (i * chunk))) + ");\n");
			}
			out.write("\t\treturn data;\n");
			out.write("\t}\n");
			// Constructor
			out.write("\t/** Constructs a new image with the data from " + srcFile.getName() + ". */\n");
			out.write("\tpublic " + removeExtension(dstFile.getName()) + "() {\n");
			out.write("\t\tsuper(getData());\n");
			out.write("\t}\n");
			// Java footer
			out.write("}");
			out.close();
		}
		catch(IOException ex){ error("An error occured while writing to the output file."); return; }
		// Alert user to success
		alert(srcPath + "\nhas been converted to\n" + dstPath);
	}
	
	// Read an input stream into a byte array
	private byte[] getStreamAsBytes(InputStream stream, int streamSize){
		int chunk = 1024;
		byte[] data = new byte[chunk];
		int b = -1;
		int size = 0;
		double progress;
		while(true){
			try{ b = stream.read(); }
			catch(IOException ex){ error(ex); return null; }
			if(b == -1) break;
			data[size] = (byte) b;
			size++;
			// Increase the size of the array
			if(size % (chunk - 1) == 0){
				byte[] biggerData = new byte[data.length + chunk];
				System.arraycopy(data, 0, biggerData, 0, data.length);
				data = biggerData;
			}
			// Progress (if GUI)
			progress = ((double)(size)) / ((double)(streamSize)) * 0.5;
			if(0.5 < progress) progress = 0.5;
			if(GUI != null) GUI.setProgress(progress);
		}
		try{ stream.close(); }
		catch(IOException ex){}
		// Resize the array to the right size
		byte[] bytes = new byte[size];
		System.arraycopy(data, 0, bytes, 0, size);
		return bytes;
	}
	
	// Get the file extension or null
	public static String getFileExtension(String file){
		if(file == null) return null;
		if(file.length() == 0) return null;
		String c;
		for(int i=file.length()-1; i>=0; i--){
			c = file.substring(i, i+1);
			if(c.equals(".")) return file.substring(i+1);
			else if(c.equals("/") || c.equals("\\")) return null;
		}
		return null;
	}
	
	// Is the file extension a supported format?
	public static boolean supported(String file){
		String extension = getFileExtension(file);
		if(extension == null) return false;
		extension = extension.toUpperCase();
		for(int i=0; i<SUPPORTED.length; i++){
			if(SUPPORTED[i].equals(extension)) return true;
		}
		return false;
	}
	
	// Get the file path and name without its extension
	public static String removeExtension(String file){
		String extension = getFileExtension(file);
		if(extension == null) return file;
		return file.substring(0, file.length() - extension.length() - 1);
	}
	
	// Convert file name to legal class name, leaving path and extension in tact
	private static String className(String file){
		// Extension
		String extension = getFileExtension(file);
		if(extension == null) extension = "";
		// Path and name
		String path = removeExtension(file);
		if(path.length() == 0) return extension;
		int i = path.length();
		String c = "";
		while(i != 0){
			i--;
			if(i != 0) c = path.substring(i, i+1);
			if(c.equals("\\")) break;
			if(c.equals("/")) break;
		}
		String name = path.substring(i+1);
		if(i != -1) path = path.substring(0, i+1);
		else path = "";		
		// Remove illegal characters from name
		String allowed = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789_-";
		String newName = "";
		for(i=0; i<name.length(); i++){
			if(allowed.indexOf(name.substring(i, i+1)) != -1) newName += name.substring(i, i+1);
		}
		if(extension != "") extension = "." + extension;
		return path + newName + extension;
	}
	
	// Get the default destination file based on source file
	public static String getDefaultDst(String src){
		return className(removeExtension(src)) + ".java";
	}
	
}
